@@ -1,4 +1,11 @@
 """This module contains the utils functions of the library."""
+
+# This file is part of Quantus.
+# Quantus is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
+# Quantus is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
+# You should have received a copy of the GNU Lesser General Public License along with Quantus. If not, see <https://www.gnu.org/licenses/>.
+# Quantus project URL: <https://github.com/understandable-machine-intelligence-lab/Quantus>.
+
 import re
 import copy
 import numpy as np
@@ -22,10 +29,17 @@ def get_superpixel_segments(img: np.ndarray, segmentation_method: str) -> np.nda
 
     Parameters
     ----------
+    img: np.ndarray
+            CxWxH image array.
+    segmentation_method: string
+        Indicates the segmentation method, i.e. "slic" or "felzenszwalb".
 
     Returns
-
+    -------
+    img: np.ndarray
+        CxWxH segmented image array.
     """
+
     if img.ndim != 3:
         raise ValueError(
             "Make sure that x is 3 dimensional e.g., (3, 224, 224) to calculate super-pixels."
@@ -56,9 +70,27 @@ def get_baseline_value(
 
     Parameters
     ----------
+    value: float, int, str, np.ndarray
+        Either the value (float, int) to fill the array with, a method (string) used to construct
+        baseline array ("mean", "uniform", "black", "white", "neighbourhood_mean" or
+        "neighbourhood_random_min_max"), or the array (np.array) to be returned.
+    arr: np.ndarray
+        CxWxH image array used to calculate baseline values, i.e. for "mean", "black" and "white" methods.
+    return_shape: tuple
+        CxWxH shape to be returned.
+    patch: np.ndarray, optional
+        CxWxH patch array to calculate baseline values.
+        Necessary for "neighbourhood_mean" and "neighbourhood_random_min_max" methods.
+    kwargs: optional
+            Keyword arguments.
 
     Returns
+    -------
+    np.ndarray
+        Baseline array in return_shape.
+
     """
+
     kwargs["return_shape"] = return_shape
     if isinstance(value, (float, int)):
         return np.full(return_shape, value)
@@ -83,7 +115,9 @@ def get_baseline_value(
                 "which will replicate the results of 'random').\n"
             )
         if value.lower() not in fill_dict:
-            raise ValueError(f"Ensure that 'value'(str) is in {list(fill_dict.keys())}")
+            raise ValueError(
+                f"Ensure that 'value'(string) is in {list(fill_dict.keys())}"
+            )
         return np.full(return_shape, fill_dict[value.lower()])
     else:
         raise ValueError("Specify 'value' as a np.array, string, integer or float.")
@@ -93,12 +127,22 @@ def get_baseline_dict(
     arr: np.ndarray, patch: Optional[np.ndarray] = None, **kwargs
 ) -> dict:
     """
-    Make a dicionary of baseline approaches depending on the input x (or patch of input).
+    Make a dictionary of baseline approaches depending on the input x (or patch of input).
 
     Parameters
     ----------
+    arr: np.ndarray
+        CxWxH image array used to calculate baseline values, i.e. for "mean", "black" and "white" methods.
+    patch: np.ndarray, optional
+        CxWxH patch array to calculate baseline values, necessary for "neighbourhood_mean" and
+        "neighbourhood_random_min_max" methods.
+    kwargs: optional
+            Keyword arguments..
 
     Returns
+    -------
+        fill_dict: dict
+            Maps all available baseline methods to baseline values.
     """
     fill_dict = {
         "mean": float(arr.mean()),
@@ -118,46 +162,93 @@ def get_baseline_dict(
     return fill_dict
 
 
-def get_name(str: str):
-    """Get the name of the class object."""
-    if str.isupper():
-        return str
-    return " ".join(re.sub(r"([A-Z])", r" \1", str).split())
+def get_name(name: str):
+    """
+    Get the name of the Metric class object.
+
+    Parameters
+    ----------
+    name: string
+        A metric name.
+
+    Returns
+    -------
+    name: string
+        A cleaned version of the Metric name.
+    """
+
+    if name.isupper():
+        return name
+    return " ".join(re.sub(r"([A-Z])", r" \1", name).split())
 
 
 def get_features_in_step(max_steps_per_input: int, input_shape: Tuple[int, ...]):
-    """Get the number of features in the iteration."""
-    return np.prod(input_shape) / max_steps_per_input
+    """
+    Get the number of features in the iteration.
+
+    Parameters
+    ----------
+    max_steps_per_input: integer
+        The number of repeated iterations on an image.
+    input_shape: tuple
+        Input shape.
+
+    Returns
+    -------
+    float
+        Product of the input shape divided by the maximum number of steps.
+    """
+    return int(np.prod(input_shape) / max_steps_per_input)
 
 
 def filter_compatible_patch_sizes(perturb_patch_sizes: list, img_size: int) -> list:
-    """Remove patch sizes that are not compatible with input size."""
+    """
+    Remove patch sizes that are not compatible with input size.
+
+    Parameters
+    ----------
+    perturb_patch_sizes: list:
+        Patch sizes for perturbation.
+    img_size: integer
+        A single dimension of an image array.
+
+    Returns
+    -------
+    list:
+        All integers within perturb_patch_sizes which are compatible with the image.
+    """
     return [i for i in perturb_patch_sizes if img_size % i == 0]
 
 
-def infer_channel_first(x: np.array):
+def infer_channel_first(x: np.array) -> bool:
     """
     Infer if the channels are first.
 
-    For 1d input:
-
-        Assumes
+    Assumes:
+        For 1d input:
             nr_channels < sequence_length
-        Returns
-            True if input shape is (nr_batch, nr_channels, sequence_length).
-            False if input shape is (nr_batch, sequence_length, nr_channels).
-            An error is raised if the two last dimensions are equal.
 
-    For 2d input:
-
-        Assumes
+        For 2d input:
             nr_channels < img_width and nr_channels < img_height
-        Returns
-            True if input shape is (nr_batch, nr_channels, img_width, img_height).
-            False if input shape is (nr_batch, img_width, img_height, nr_channels).
-            An error is raised if the three last dimensions are equal.
 
     For higher dimensional input an error is raised.
+
+    Parameters
+    ----------
+    x: np.ndarray
+         Input image.
+
+    Returns
+    -------
+    For 1d input:
+        True if input shape is (nr_batch, nr_channels, sequence_length).
+        False if input shape is (nr_batch, sequence_length, nr_channels).
+        An error is raised if the two last dimensions are equal.
+
+    For 2d input:
+        True if input shape is (nr_batch, nr_channels, img_width, img_height).
+        False if input shape is (nr_batch, img_width, img_height, nr_channels).
+        An error is raised if the three last dimensions are equal.
     """
     err_msg = "Ambiguous input shape. Cannot infer channel-first/channel-last order."
 
@@ -188,8 +279,13 @@ def make_channel_first(x: np.array, channel_first=False):
 
     Parameters
     ----------
+    x: np.ndarray
+         The input image.
 
     Returns
+    -------
+    np.ndarray
+        Image in CxWxH format.
     """
     if channel_first:
         return x
@@ -205,7 +301,19 @@ def make_channel_first(x: np.array, channel_first=False):
 
 
 def make_channel_last(x: np.array, channel_first=True):
-    """Reshape batch to channel last."""
+    """
+    Reshape batch to channel last.
+
+    Parameters
+    ----------
+    x: np.ndarray
+         The input image.
+
+    Returns
+    -------
+    np.ndarray
+        Image in WxHxC format.
+    """
     if not channel_first:
         return x
 
@@ -224,39 +332,47 @@ def get_wrapped_model(
     channel_first: bool,
     softmax: bool,
     device: Optional[str] = None,
-    predict_kwargs: Optional[Dict[str, Any]] = None,
+    model_predict_kwargs: Optional[Dict[str, Any]] = None,
 ) -> ModelInterface:
     """
     Identifies the type of a model object and wraps the model in an appropriate interface.
 
     Parameters
     ----------
-    model: a pytorch or tensorflow model that is to be wrapped.
-    channel_first (boolean): Indicates if model expects channel first or channel last layout.
-    predict_kwargs (dict, optional): Keyword arguments to be passed to the model's predict method, default = {}
+    model: Union[torch.nn.Module, tf.keras.Model]
+        A model this will be wrapped in the ModelInterface:
+    channel_first: boolean, optional
+         Indicates of the image dimensions are channel first, or channel last. Inferred from the input shape if None.
+    softmax: boolean
+        Indicates whether to use softmax probabilities or logits in model prediction. This is used for this __call__ only and won't be saved as attribute. If None, self.softmax is used.
+    device: string
+        Indicated the device on which a torch.Tensor is or will be allocated: "cpu" or "gpu".
+    model_predict_kwargs: dict, optional
+        Keyword arguments to be passed to the model's predict method.
 
     Returns
     -------
-    model (ModelInterface): A wrapped ModelInterface model.
+    model: ModelInterface
+        A wrapped ModelInterface model.
     """
-    if isinstance(model, tf.keras.Model):
-        return TensorFlowModel(
-            model=model,
-            channel_first=channel_first,
-            softmax=softmax,
-            predict_kwargs=predict_kwargs,
-        )
-    if isinstance(model, torch.nn.modules.module.Module):
-        return PyTorchModel(
-            model=model,
-            channel_first=channel_first,
-            softmax=softmax,
-            device=device,
-            predict_kwargs=predict_kwargs,
-        )
-    raise ValueError(
-        "Model needs to be tf.keras.Model or torch.nn.modules.module.Module."
-    )
+    if util.find_spec("tensorflow"):
+        if isinstance(model, tf.keras.Model):
+            return TensorFlowModel(
+                model=model,
+                channel_first=channel_first,
+                softmax=softmax,
+                model_predict_kwargs=model_predict_kwargs,
+            )
+    if util.find_spec("torch"):
+        if isinstance(model, torch.nn.Module):
+            return PyTorchModel(
+                model=model,
+                channel_first=channel_first,
+                softmax=softmax,
+                device=device,
+                model_predict_kwargs=model_predict_kwargs,
+            )
+    raise ValueError("Model needs to be tf.keras.Model or torch.nn.Module.")
 
 
 def blur_at_indices(
@@ -266,11 +382,24 @@ def blur_at_indices(
     indexed_axes: Sequence[int],
 ) -> np.array:
     """
-    Returns a version of arr that is blurred at indices.
+    Creates a version of arr that is blurred at indices.
 
     Parameters
     ----------
+    arr: np.ndarray
+         Array to be perturbed.
+    kernel: np.ndarray
+         Kernel used for blurring.
+    indices: int, sequence, tuple
+        Array-like, with a subset shape of arr.
+    indexed_axes: sequence
+        The dimensions of arr that are indexed. These need to be consecutive and either
+         include the first or last dimension of array.
 
+    Returns
+    -------
+    np.ndarray
+        A version of arr that is blurred at indices.
     """
 
     assert kernel.ndim == len(
@@ -333,13 +462,21 @@ def blur_at_indices(
 
 def create_patch_slice(
     patch_size: Union[int, Sequence[int]], coords: Sequence[int]
-) -> Tuple[np.ndarray]:
+) -> Tuple[slice]:
     """
     Create a patch slice from patch size and coordinates.
 
     Parameters
     ----------
+    patch_size: int, sequence
+        One- or multidimensional patch size.
+    coords: sequence
+        Coordinates for creating patches.
 
+    Returns
+    -------
+    tuple
+        Patches at all provided coordinates.
     """
 
     if isinstance(patch_size, int):
@@ -362,11 +499,10 @@ def create_patch_slice(
     # make sure that each element in tuple is integer
     patch_size = tuple(int(patch_size_dim) for patch_size_dim in patch_size)
 
-    gridcoords = [
-        np.arange(coord, coord + patch_size_dim)
+    patch_slice = [
+        slice(coord, coord + patch_size_dim)
         for coord, patch_size_dim in zip(coords, patch_size)
     ]
-    patch_slice = np.meshgrid(*gridcoords)
 
     return tuple(patch_slice)
 
@@ -379,7 +515,18 @@ def get_nr_patches(
 
     Parameters
     ----------
+        patch_size: int, sequence
+            One- or multidimensional patch size.
+        shape (shape: Tuple[int, ...]): The image shape.
+        overlap: boolean
+            Indicates whether overlapping patches is used or not.
+
+    Returns
+    -------
+    integer
+        Number of patches that fit into the image.
     """
+
     if isinstance(patch_size, int):
         patch_size = (patch_size,)
     patch_size = np.array(patch_size)
@@ -403,12 +550,25 @@ def _pad_array(
     pad_width: Union[int, Sequence[int], Sequence[Tuple[int]]],
     mode: str,
     padded_axes: Sequence[int],
-):
+) -> np.array:
     """
-    To allow for any patch_size we add padding to the array.
+    Pad an array according to the mode.
 
     Parameters
     ----------
+    arr: np.ndarray
+         Array to pad.
+    pad_width: int, sequence
+            The size of the padding.
+    mode: string
+        numpy.pad mode (see https://numpy.org/doc/stable/reference/generated/numpy.pad.html)
+    padded_axes: sequence
+        Indices of axes to pad.
+
+    Returns
+    -------
+    np.ndarray
+        Padded array.
     """
 
     assert (
@@ -424,6 +584,7 @@ def _pad_array(
                 assert len(p) == 2, "Elements in pad_width need to have length 2"
 
     pad_width_list = []
+
     for ax in range(arr.ndim):
         if ax not in padded_axes:
             pad_width_list.append((0, 0))
@@ -438,7 +599,9 @@ def _pad_array(
             )
         else:
             pad_width_list.append(pad_width[[p for p in padded_axes].index(ax)])
+
     arr_pad = np.pad(arr, pad_width_list, mode=mode)
+
     return arr_pad
 
 
@@ -452,12 +615,17 @@ def _unpad_array(
 
     Parameters
     ----------
-        arr: a numpy array of the input to be unpaded
-        pad_witdh: the width of the padding for the different dimensions
-        padded_axes: the axes for padding
+    arr: np.ndarray
+         A numpy array of the input to be unpaded.
+    pad_witdh: int, sequence
+        The width of the padding for the different dimensions.
+    padded_axes: sequence
+        The axes for padding.
 
     Returns
-        the unpadded array
+    -------
+        np.ndarray:
+        The unpadded array.
 
     """
 
@@ -502,8 +670,15 @@ def expand_attribution_channel(a_batch: np.ndarray, x_batch: np.ndarray):
 
     Parameters
     ----------
-        x_batch: a np.ndarray which contains the input data that are explained
-        a_batch: a Union[np.ndarray, None] which contains pre-computed attributions i.e., explanations
+    x_batch: np.ndarray
+        A np.ndarray which contains the input data that are explained.
+    a_batch: np.ndarray
+        An array which contains pre-computed attributions i.e., explanations.
+
+    Returns
+    -------
+    np.ndarray
+        A x_batch with dimensions matching those of a_batch.
     """
     if a_batch.shape[0] != x_batch.shape[0]:
         raise ValueError(
@@ -532,11 +707,15 @@ def infer_attribution_axes(a_batch: np.ndarray, x_batch: np.ndarray) -> Sequence
 
     Parameters
     ----------
-        x_batch: a np.ndarray which contains the input data that are explained
-        a_batch: a Union[np.ndarray, None] which contains pre-computed attributions i.e., explanations
+    x_batch: np.ndarray
+        A np.ndarray which contains the input data that are explained.
+    a_batch: np.ndarray
+        An array which contains pre-computed attributions i.e., explanations.
 
     Returns
-        the axes inferred
+    -------
+    np.ndarray
+        The axes inferred.
     """
     # TODO: Adapt for batched processing.
 
@@ -630,15 +809,24 @@ def expand_indices(
 ) -> Tuple:
     """
     Expands indices to fit array shape. Returns expanded indices.
+        --> if indices are a sequence of ints, they are interpreted as indices to the flattened arr,
+            and subsequently expanded
+        --> if indices contains only slices and 1d sequences for arr, everything is interpreted as slices
+        --> if indices contains already expanded indices, they are returned as is
 
     Parameters
     ----------
-        arr: the input to the expanded
-        indicies: list of indicies
-        indexed_axes: refers to all axes that are not indexed by slice(None).
+    arr: np.ndarray
+        The input to the expanded.
+    indices: int, sequence, tuple
+        A list of indices.
+    indexed_axes: sequence
+        Refers to all axes that are not indexed by slice(None).
 
     Returns
-
+    -------
+    tuple
+        Expanded indices.
     """
     # TODO: Adapt for batched processing.
 
@@ -651,14 +839,15 @@ def expand_indices(
         expanded_indices = [indices]
     else:
         expanded_indices = []
-        for idx in indices:
+        for i, idx in enumerate(indices):
             if isinstance(idx, slice) and idx == slice(None):
                 pass
             elif isinstance(idx, slice):
                 start = idx.start
-                end = idx.end
+                end = idx.stop
                 step = idx.step
-                expanded_indices.append(np.arange(start, end, step))
+                tmp = np.arange(start, end, step)
+                expanded_indices.append(tmp)
             elif isinstance(idx, np.ndarray):
                 expanded_indices.append(idx)
             else:
@@ -668,13 +857,24 @@ def expand_indices(
                     raise ValueError("Unsupported type of indices.")
 
     # Check if unraveling is needed.
-    if np.all([isinstance(i, int) for i in expanded_indices]):
+    if np.all([isinstance(idx, int) for idx in expanded_indices]):
         expanded_indices = np.unravel_index(
             expanded_indices, tuple([arr.shape[i] for i in indexed_axes])
         )
+    elif not np.all(
+        [
+            isinstance(idx, np.ndarray) and idx.ndim == len(expanded_indices)
+            for idx in expanded_indices
+        ]
+    ):
+        # Meshgrid sliced axes to account for correct slicing. Correct switched first two axes by meshgrid
+        expanded_indices = [
+            np.swapaxes(idx, 0, 1) if idx.ndim > 1 else idx
+            for idx in np.meshgrid(*expanded_indices)
+        ]
 
     # Handle case of 1D indices.
-    if not np.array(expanded_indices).ndim > 1:
+    if np.all([isinstance(idx, int) for idx in expanded_indices]):
         expanded_indices = [np.array(expanded_indices)]
 
     # Cast to list so item assignment works.
@@ -688,7 +888,8 @@ def expand_indices(
     for i in range(len(expanded_indices)):
         if expanded_indices[i].ndim != len(expanded_indices):
             expanded_indices[i] = np.expand_dims(
-                expanded_indices[i], axis=tuple(range(len(expanded_indices) - 1))
+                expanded_indices[i],
+                axis=tuple(range(len(expanded_indices) - expanded_indices[i].ndim)),
             )
 
     # Buffer with None-slices if indices index the last axes.
@@ -704,11 +905,15 @@ def get_leftover_shape(arr: np.array, axes: Sequence[int]) -> Tuple:
 
     Parameters
     ----------
-        arr: the input to the expanded
-        axes: a sequence of ints containing the axes
+    arr: np.ndarray
+         The input to the expanded.
+    axes: sequence
+        A sequence of ints containing the axes.
 
     Returns
-        a tuple of the leftover shape
+    -------
+    leftover_shape: tuple
+        The leftover shape.
     """
 
     # TODO: Adapt for batched processing.
@@ -721,17 +926,22 @@ def get_leftover_shape(arr: np.array, axes: Sequence[int]) -> Tuple:
 
 def offset_coordinates(indices: list, offset: tuple, img_shape: tuple):
     """
-    Checks if offset coordinates are within the image frame
-    Based on https://github.com/tleemann/road_evaluation.
-      Parameters
+    Checks if offset coordinates are within the image frame.
+        Adapted from: https://github.com/tleemann/road_evaluation.
+
+    Parameters
     ----------
-        a tuple of the leftover shape
-        indices (list): list of indices to be offset.
-        offset (tuple): offset for the coordinates, e.g. offset (1,1) adds 1 to both coordinates.
-        img_shape (tuple): image shape in (channels, height, width) format.
+    indices: list
+        List of indices to be offset.
+    offset: tuple
+        Offset for the coordinates, e.g. offset (1,1) adds 1 to both coordinates.
+    img_shape: tuple
+        Image shape in (channels, height, width) format.
 
     Returns
-         offset coordinates for valid indices and the list of booleans which identifies valid ids.
+    -------
+    list
+        Offset coordinates for valid indices and the list of booleans which identifies valid ids.
     """
     x = indices // img_shape[2]
     y = indices % img_shape[2]
@@ -743,5 +953,19 @@ def offset_coordinates(indices: list, offset: tuple, img_shape: tuple):
 
 
 def calculate_auc(values: np.array, dx: int = 1.0):
-    """Calculate area under the curve using the composite trapezoidal rule."""
+    """
+    Calculate area under the curve using the composite trapezoidal rule.
+
+    Parameters
+    ----------
+    values: np.ndarray
+         Input array.
+    dx: integer
+        The spacing between sample points. The default is 1.
+
+    Returns
+    -------
+    np.ndarray
+        Definite integral of values.
+    """
     return np.trapz(np.array(values), dx=dx)
