@@ -1,4 +1,11 @@
-"""This module implements the base class for creating evaluation measures."""
+"""This module implements the base class for creating evaluation metrics."""
+
+# This file is part of Quantus.
+# Quantus is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
+# Quantus is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
+# You should have received a copy of the GNU Lesser General Public License along with Quantus. If not, see <https://www.gnu.org/licenses/>.
+# Quantus project URL: <https://github.com/understandable-machine-intelligence-lab/Quantus>.
+
 import warnings
 from abc import abstractmethod
 from typing import Any, Callable, Dict, List, Optional, Tuple, Union
@@ -40,24 +47,34 @@ class Metric:
         A child metric can benefit from the following class methods:
         - __call__(): Will call general_preprocess(), apply () on each
                       instance and finally call custom_preprocess().
-                      To use this method the child Metric needs to implement
-                      ().
+                      To use this method the child Metric needs to implement ().
         - general_preprocess(): Prepares all necessary data structures for evaluation.
                                 Will call custom_preprocess() at the end.
 
         Parameters
         ----------
-        abs (boolean): Indicates whether absolute operation is applied on the attribution.
-        normalise (boolean): Indicates whether normalise operation is applied on the attribution.
-        normalise_func (callable): Attribution normalisation function applied in case normalise=True.
-        normalise_func_kwargs (dict): Keyword arguments to be passed to normalise_func on call.
-        return_aggregate (boolean): Indicates if an aggregated score should be computed over all instances.
-        aggregate_func (callable): Callable that aggregates the scores given an evaluation call..
-        default_plot_func (callable): Callable that plots the metrics result.
-        disable_warnings (boolean): Indicates whether the warnings are printed.
-        display_progressbar (boolean): Indicates whether a tqdm-progress-bar is printed.
-
+        abs: boolean
+            Indicates whether absolute operation is applied on the attribution.
+        normalise: boolean
+            Indicates whether normalise operation is applied on the attribution.
+        normalise_func: callable
+            Attribution normalisation function applied in case normalise=True.
+        normalise_func_kwargs: dict
+            Keyword arguments to be passed to normalise_func on call.
+        return_aggregate: boolean
+            Indicates if an aggregated score should be computed over all instances.
+        aggregate_func: callable
+            Callable that aggregates the scores given an evaluation call..
+        default_plot_func: callable
+            Callable that plots the metrics result.
+        disable_warnings: boolean
+            Indicates whether the warnings are printed.
+        display_progressbar: boolean
+            Indicates whether a tqdm-progress-bar is printed.
+        kwargs: optional
+            Keyword arguments.
         """
+
         # Run deprecation warnings.
         warn_func.deprecation_warnings(kwargs)
         warn_func.check_kwargs(kwargs)
@@ -93,8 +110,8 @@ class Metric:
         explain_func_kwargs: Optional[Dict[str, Any]],
         model_predict_kwargs: Optional[Dict],
         softmax: Optional[bool],
-        custom_batch: Optional[Any] = None,
         device: Optional[str] = None,
+        custom_batch: Optional[Any] = None,
         **kwargs,
     ) -> Union[int, float, list, dict, None]:
         """
@@ -108,50 +125,72 @@ class Metric:
 
         Parameters
         ----------
-        model: a torch model e.g., torchvision.models that is subject to explanation
-        x_batch: a np.ndarray which contains the input data that are explained
-        y_batch: a np.ndarray which contains the output labels that are explained
-        a_batch: a Union[np.ndarray, None] which contains pre-computed attributions i.e., explanations
-        s_batch: a Union[np.ndarray, None] which contains segmentation masks that matches the input
-        custom_batch (Any): Gives flexibility ot the user to use for evaluation, can hold any variable.
-        channel_first (boolean, optional): Indicates of the image dimensions are channel first, or channel last.
+        model: Union[torch.nn.Module, tf.keras.Model]
+            A torch or tensorflow model that is subject to explanation.
+        x_batch: np.ndarray
+            A np.ndarray which contains the input data that are explained.
+        y_batch: np.ndarray
+            A np.ndarray which contains the output labels that are explained.
+        a_batch: np.ndarray, optional
+            A np.ndarray which contains pre-computed attributions i.e., explanations.
+        s_batch: np.ndarray, optional
+            A np.ndarray which contains segmentation masks that matches the input.
+        channel_first: boolean, optional
+            Indicates of the image dimensions are channel first, or channel last.
             Inferred from the input shape if None.
-        explain_func (callable): Callable generating attributions.
-        explain_func_kwargs (dict, optional): Keyword arguments to be passed to explain_func on call.
-        model_predict_kwargs (dict, optional): Keyword arguments to be passed to the model's predict method.
-        device (string): Indicated the device on which a torch.Tensor is or will be allocated: "cpu" or "gpu".
-        softmax (boolean): Indicates whether to use softmax probabilities or logits in model prediction.
-            This is used for this __call__ only and won't be saved as attribute. If None, self.softmax is used.
+        explain_func: callable
+            Callable generating attributions.
+        explain_func_kwargs: dict, optional
+            Keyword arguments to be passed to explain_func on call.
+        model_predict_kwargs: dict, optional
+            Keyword arguments to be passed to the model's predict method.
+        softmax: boolean
+            Indicates whether to use softmax probabilities or logits in model prediction.
+                This is used for this __call__ only and won't be saved as attribute. If None, self.softmax is used.
+        device: string
+            Indicated the device on which a torch.Tensor is or will be allocated: "cpu" or "gpu".
+        custom_batch: any
+            Any object that can be passed to the evaluation process.
+            Gives flexibility to the user to adapt for implementing their own metric.
+        kwargs: optional
+            Keyword arguments.
 
         Returns
         -------
-        last_results: a list of float(s) with the evaluation outcome of concerned batch.
+        last_results: list
+            a list of Any with the evaluation scores of the concerned batch.
 
-        Examples
+        Examples:
         --------
-        # Enable GPU.
-        >> device = torch.device("cuda:0" if torch.cuda.is_available() else "cpu")
+            # Minimal imports.
+            >> import quantus
+            >> from quantus import LeNet
+            >> import torch
 
-        # Load a pre-trained LeNet classification model (architecture at quantus/helpers/models).
-        >> model = LeNet()
-        >> model.load_state_dict(torch.load("tutorials/assets/mnist"))
+            # Enable GPU.
+            >> device = torch.device("cuda:0" if torch.cuda.is_available() else "cpu")
 
-        # Load MNIST datasets and make loaders.
-        >> test_set = torchvision.datasets.MNIST(root='./sample_data', download=True)
-        >> test_loader = torch.utils.data.DataLoader(test_set, batch_size=24)
+            # Load a pre-trained LeNet classification model (architecture at quantus/helpers/models).
+            >> model = LeNet()
+            >> model.load_state_dict(torch.load("tutorials/assets/pytests/mnist_model"))
 
-        # Load a batch of inputs and outputs to use for XAI evaluation.
-        >> x_batch, y_batch = iter(test_loader).next()
-        >> x_batch, y_batch = x_batch.cpu().numpy(), y_batch.cpu().numpy()
+            # Load MNIST datasets and make loaders.
+            >> test_set = torchvision.datasets.MNIST(root='./sample_data', download=True)
+            >> test_loader = torch.utils.data.DataLoader(test_set, batch_size=24)
 
-        # Generate Saliency attributions of the test set batch of the test set.
-        >> a_batch_saliency = Saliency(model).attribute(inputs=x_batch, target=y_batch, abs=True).sum(axis=1)
-        >> a_batch_saliency = a_batch_saliency.cpu().numpy()
+            # Load a batch of inputs and outputs to use for XAI evaluation.
+            >> x_batch, y_batch = iter(test_loader).next()
+            >> x_batch, y_batch = x_batch.cpu().numpy(), y_batch.cpu().numpy()
 
-        # Initialise the metric and evaluate explanations by calling the metric instance.
-        >> metric = Metric(abs=True, normalise=False)
-        >> scores = metric(model=model, x_batch=x_batch, y_batch=y_batch, a_batch=a_batch_saliency}
+            # Generate Saliency attributions of the test set batch of the test set.
+            >> a_batch_saliency = Saliency(model).attribute(inputs=x_batch, target=y_batch, abs=True).sum(axis=1)
+            >> a_batch_saliency = a_batch_saliency.cpu().numpy()
+
+            # Initialise the metric and evaluate explanations by calling the metric instance.
+            >> metric = Metric(abs=True, normalise=False)
+            >> scores = metric(model=model, x_batch=x_batch, y_batch=y_batch, a_batch=a_batch_saliency}
         """
+
         # Run deprecation warnings.
         warn_func.deprecation_warnings(kwargs)
         warn_func.check_kwargs(kwargs)
@@ -170,13 +209,13 @@ class Metric:
             y_batch=y_batch,
             a_batch=a_batch,
             s_batch=s_batch,
-            custom_batch=custom_batch,
             channel_first=channel_first,
             explain_func=explain_func,
             explain_func_kwargs=explain_func_kwargs,
             model_predict_kwargs=model_predict_kwargs,
             softmax=softmax,
             device=device,
+            custom_batch=custom_batch,
         )
 
         # Create progress bar if desired.
@@ -258,9 +297,28 @@ class Metric:
         p: Optional[np.ndarray] = None,
     ) -> Any:
         """
+        Evaluate instance gets model and data for a single instance as input and returns the evaluation result.
+
         This method needs to be implemented to use __call__().
 
-        Gets model and data for a single instance as input, returns result.
+        Parameters
+        ----------
+        i: integer
+            The evaluation instance.
+        model: ModelInterface
+            A ModelInteface that is subject to explanation.
+        x: np.ndarray
+            The input to be evaluated on an instance-basis.
+        y: np.ndarray
+            The output to be evaluated on an instance-basis.
+        a: np.ndarray
+            The explanation to be evaluated on an instance-basis.
+        s: np.ndarray
+            The segmentation to be evaluated on an instance-basis.
+        c: any
+            The custom input to be evaluated on an instance-basis.
+        p: any
+            The custom preprocess input to be evaluated on an instance-basis.
         """
         raise NotImplementedError()
 
@@ -271,28 +329,68 @@ class Metric:
         y_batch: Optional[np.ndarray],
         a_batch: Optional[np.ndarray],
         s_batch: Optional[np.ndarray],
-        custom_batch: Optional[np.ndarray],
         channel_first: Optional[bool],
         explain_func: Optional[Callable],
         explain_func_kwargs: Optional[Dict[str, Any]],
         model_predict_kwargs: Optional[Dict],
         softmax: bool,
         device: Optional[str],
+        custom_batch: Optional[np.ndarray],
     ) -> Tuple[
         ModelInterface, np.ndarray, np.ndarray, np.ndarray, np.ndarray, Any, Any
     ]:
         """
         Prepares all necessary variables for evaluation.
 
-        - Reshapes data to channel first layout.
-        - Wraps model into ModelInterface.
-        - Creates attributions if necessary.
-        - Expands attributions to data shape (adds channel dimension).
-        - Calls custom_preprocess().
-        - Normalises attributions if desired.
-        - Takes absolute of attributions if desired.
-        - If no segmentation s_batch given, creates list of Nones with as many
-          elements as there are data instances.
+            - Reshapes data to channel first layout.
+            - Wraps model into ModelInterface.
+            - Creates attributions if necessary.
+            - Expands attributions to data shape (adds channel dimension).
+            - Calls custom_preprocess().
+            - Normalises attributions if desired.
+            - Takes absolute of attributions if desired.
+            - If no segmentation s_batch given, creates list of Nones with as many
+              elements as there are data instances.
+            - If no custom_batch given, creates list of Nones with as many
+              elements as there are data instances.
+
+        Parameters
+        ----------
+
+        model: Union[torch.nn.Module, tf.keras.Model]
+            A torch or tensorflow model e.g., torchvision.models that is subject to explanation.
+        x_batch: np.ndarray
+            A np.ndarray which contains the input data that are explained.
+        y_batch: np.ndarray
+            A np.ndarray which contains the output labels that are explained.
+        a_batch: np.ndarray, optional
+            A np.ndarray which contains pre-computed attributions i.e., explanations.
+        s_batch: np.ndarray, optional
+            A np.ndarray which contains segmentation masks that matches the input.
+        channel_first: boolean, optional
+            Indicates of the image dimensions are channel first, or channel last.
+            Inferred from the input shape if None.
+        explain_func: callable
+            Callable generating attributions.
+        explain_func_kwargs: dict, optional
+            Keyword arguments to be passed to explain_func on call.
+        model_predict_kwargs: dict, optional
+            Keyword arguments to be passed to the model's predict method.
+        softmax: boolean
+            Indicates whether to use softmax probabilities or logits in model prediction.
+            This is used for this __call__ only and won't be saved as attribute. If None, self.softmax is used.
+        device: string
+            Indicated the device on which a torch.Tensor is or will be allocated: "cpu" or "gpu".
+        custom_batch: any
+            Gives flexibility ot the user to use for evaluation, can hold any variable.
+        kwargs: optional
+            Keyword arguments.
+
+        Returns
+        -------
+        tuple
+            A general preprocess.
+
         """
 
         # Reshape input batch to channel first order:
@@ -309,7 +407,7 @@ class Metric:
                 channel_first=channel_first,
                 softmax=softmax,
                 device=device,
-                predict_kwargs=model_predict_kwargs,
+                model_predict_kwargs=model_predict_kwargs,
             )
 
         # Save as attribute, some metrics need it during processing.
@@ -374,7 +472,7 @@ class Metric:
         if self.abs:
             a_batch = np.abs(a_batch)
 
-        # This is needed for iterator (zipped over x_batch, y_batch, a_batch, s_batch, custom_batch)
+        # This is needed for iterator (zipped over x_batch, y_batch, a_batch, s_batch, custom_batch).
         if s_batch is None:
             s_batch = [None for _ in x_batch]
         if custom_batch is None:
@@ -401,9 +499,35 @@ class Metric:
     ) -> Tuple[
         ModelInterface, np.ndarray, np.ndarray, np.ndarray, np.ndarray, Any, Any
     ]:
-        """s
+        """
         Implement this method if you need custom preprocessing of data,
         model alteration or simply for creating/initialising additional attributes.
+
+        If this function is implemented for your metric, make sure to add in the metric-specific
+        function custom_preprocess():
+
+        >> custom_preprocess_batch = [None for _ in x_batch]
+
+        Parameters
+        ----------
+        model: Union[torch.nn.Module, tf.keras.Model]
+            A torch or tensorflow model e.g., torchvision.models that is subject to explanation.
+        x_batch: np.ndarray
+            A np.ndarray which contains the input data that are explained.
+        y_batch: np.ndarray
+            A np.ndarray which contains the output labels that are explained.
+        a_batch: np.ndarray, optional
+            A np.ndarray which contains pre-computed attributions i.e., explanations.
+        s_batch: np.ndarray, optional
+            A np.ndarray which contains segmentation masks that matches the input.
+        custom_batch: any
+            Gives flexibility ot the user to use for evaluation, can hold any variable.
+
+        Returns
+        -------
+        tuple
+            In addition to the x_batch, y_batch, a_batch, s_batch and custom_batch,
+            returning a custom preprocess batch (custom_preprocess_batch).
         """
         custom_preprocess_batch = [None for _ in x_batch]
         return (
@@ -428,6 +552,26 @@ class Metric:
         """
         Implement this method if you need custom postprocessing of results or
         additional attributes.
+
+        Parameters
+        ----------
+        model: Union[torch.nn.Module, tf.keras.Model]
+            A torch or tensorflow model e.g., torchvision.models that is subject to explanation.
+        x_batch: np.ndarray
+            A np.ndarray which contains the input data that are explained.
+        y_batch: np.ndarray
+            A np.ndarray which contains the output labels that are explained.
+        a_batch: np.ndarray, optional
+            A np.ndarray which contains pre-computed attributions i.e., explanations.
+        s_batch: np.ndarray, optional
+            A np.ndarray which contains segmentation masks that matches the input.
+        custom_batch: any
+            Gives flexibility ot the user to use for evaluation, can hold any variable.
+
+        Returns
+        -------
+        any
+            Can be implemented, optionally by the user.
         """
         pass
 
@@ -445,14 +589,20 @@ class Metric:
 
         Parameters
         ----------
-        plot_func: a Callable with the actual plotting logic.
-        show: a boolean to state if the plot shall be shown.
-        path_to_save: a string that specifies the path to save file.
-        args: an optional with additional arguments.
-        kwargs: an optional dict with additional arguments.
-
-        Returns: None.
+        plot_func: callable
+            A Callable with the actual plotting logic.
+        show: boolean
+            A boolean to state if the plot shall be shown.
+        path_to_save (str):
+            A string that specifies the path to save file.
+        args: optional
+            An optional with additional arguments.
+        kwargs: optional
+            An optional dict with additional arguments.
 
+        Returns
+        -------
+        None
         """
         # Get plotting func if not provided.
         if plot_func is None:
@@ -483,9 +633,11 @@ class Metric:
     def get_params(self) -> dict:
         """
         List parameters of metric.
-        Returns: a dictionary with attributes if not excluded from pre-determined list
-        -------
 
+        Returns
+        -------
+        dict
+            A dictionary with attributes if not excluded from pre-determined list.
         """
         attr_exclude = [
             "args",
@@ -523,6 +675,36 @@ class PerturbationMetric(Metric):
         display_progressbar: bool,
         **kwargs,
     ):
+        """
+        Initialise the PerturbationMetric base class.
+
+        Parameters
+        ----------
+        abs: boolean
+            Indicates whether absolute operation is applied on the attribution.
+        normalise: boolean
+            Indicates whether normalise operation is applied on the attribution.
+        normalise_func: callable
+            Attribution normalisation function applied in case normalise=True.
+        normalise_func_kwargs: dict
+            Keyword arguments to be passed to normalise_func on call.
+        perturb_func: callable
+            Input perturbation function.
+        perturb_func_kwargs: dict
+            Keyword arguments to be passed to perturb_func, default={}.
+        return_aggregate: boolean
+            Indicates if an aggregated score should be computed over all instances.
+        aggregate_func: callable
+            Callable that aggregates the scores given an evaluation call..
+        default_plot_func: callable
+            Callable that plots the metrics result.
+        disable_warnings: boolean
+            Indicates whether the warnings are printed.
+        display_progressbar: boolean
+            Indicates whether a tqdm-progress-bar is printed.
+        kwargs: optional
+            Keyword arguments.
+        """
 
         # Initialise super-class with passed parameters.
         super().__init__(
