@@ -230,7 +230,12 @@ def test_normalise_by_negative(
 def test_denormalise(
     data: np.ndarray, params: dict, expected: Union[float, dict, bool]
 ):
-    out = denormalise(img=data, **params)
+    out = denormalise(
+        a=data,
+        mean=np.array([0.485, 0.456, 0.406]),
+        std=np.array([0.229, 0.224, 0.225]),
+        **params,
+    )
     assert np.all(
         o == e for o, e in zip(np.array(out).flatten(), np.array(expected).flatten())
     ), "Test failed."
